/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if defined(__linux__) || defined(__sun__) || defined(BSD4_4)
#define _MDH_HAS_CPU    1
#define _MDH_HAS_DISK   1
#define _MDH_HAS_UPTIME 1
#endif

#if defined(__linux__) || defined(__sun__)
#define _MDH_HAS_MEM    1
#define _MDH_HAS_NET    1
#endif

#define _MDH_URL       "http://www.gozer.org/my_stuff/gtk/mdh.php"
#define _MDH_EMAIL     "mdh at gozer dot org"

/* errors for panel_t->errno, used to keep error log dupes out */
#define _ERR_NONE      0       /* success */
#define _ERR_EXIST    -1       /* file/dir or network interface not found */
#define _ERR_EMPTY    -2       /* empty value */
#define _ERR_ILEN     -3       /* invalid station-id length */

/* maximum length of data that can be displayed in the panel */
#define _DATA_MAX     100

/* max scratch-pads (starts at 0, 5 by default) */
#define _MDH_MAX_PAD  4

/* max user-defineable buttons (starts at 0, 10 by default) */
#define _MDH_MAX_CUST 9

/* max mailboxes to be monitored (starts at 0, 3 by default) */
#define _MDH_MAX_MAIL 2

/* max disks to be monitored (starts at 0, 3 by default) */
#define _MDH_MAX_DISK 2

#ifdef __sun__
# define _MDH_MAX_NET 0
#else
# define _MDH_MAX_NET 2
#endif

#define _MAIL_I_MIN   5        /* min mail interval */
#define _MAIL_I_DEF   30       /* default mail interval */
#define _MAIL_I_MAX   300      /* max mail interval */

/* panel types */
enum {
    _PANEL_TYPE_TIME,
    _PANEL_TYPE_MAIL,
#if _MDH_HAS_CPU
    _PANEL_TYPE_CPU,
#endif
#if _MDH_HAS_MEM
    _PANEL_TYPE_MEM,
#endif
#if _MDH_HAS_DISK
    _PANEL_TYPE_DISK,
#endif
#if _MDH_HAS_NET
    _PANEL_TYPE_NET,
#endif
    _PANEL_TYPE_TEMP,
#if _MDH_HAS_UPTIME
    _PANEL_TYPE_UPTIME,
#endif
    _PANEL_TYPE_N
};

struct toolbar {
    gboolean     debug;        /* enable debugging */

    GtkWidget    *window;

    gint         geom_x;
    gint         geom_y;

    gboolean     on_top;       /* window on top */

    gboolean     tooltips;     /* use tooltips */

    gboolean     hidden;       /* toolbar (most of it) is toggled hidden */

    gchar        *run_item;    /* xterm to use (Run in terminal) */
    gboolean     run_save;     /* save run history */
    gboolean     run_close;    /* close run window after execution */

    GtkTooltips  *tips;        /* contains panel tooltip data */

    gchar        *font;        /* font to use for main toolbar widgets */

    gboolean     config_open;  /* used to disallow multiple windows open */
    gboolean     error_open;
    gboolean     custom_open;
    gboolean     pad_open;
    gboolean     run_open;

    GtkWidget    *menu;        /* popup (main) menu widget */

    GSList       *buttons;     /* contains a list of panel buttons */
    GSList       *custom;      /* contains a list of custom buttons */
    GSList       *panels;      /* contains a list of panels */

    GSList       *errors;      /* error log */

    GList        *history;     /* run history */
};

struct button_t {
    GtkWidget    *box;         /* hbox used for show/hide_all() */
    GtkWidget    *widget;      /* button */

    GtkWidget    *cfg_w;       /* config widgets (toggle insensitive) */

    const gchar *name;
    const gchar *tooltip;

    GtkFunction  callback;     /* click action */

    gboolean     hidden;

    gboolean     can_toggle;   /* allow mdh_toolbar_toggle_all() to hide? */

    gboolean     enabled;
};

struct custom_t {
    GtkWidget    *box;         /* hbox used for show/hide_all() */
    GtkWidget    *widget;      /* label */

    GtkFunction  callback;     /* click action */

    gboolean     hidden;

    gboolean     enabled;

    gchar        *value;
    gchar        *command;
    gchar        *tooltip;
};

struct panel_t {
    GtkWidget    *event_box;   /* handles the click events */
    GtkWidget    *box;         /* hbox used for show/hide_all() */
    GtkWidget    *widget;      /* label */

    GtkWidget    *cfg_w[8];    /* config widgets (toggle insensitive) */

    GtkFunction  callback;     /* click action */
    GtkFunction  function;     /* timer action */

    gboolean     init;

    gchar        *name;        /* name of the panel (time, mail, cpu, etc.) */

    GtkFunction  init_open;
    GtkFunction  init_close;

    gint         type;         /* _TYPE_MAIL, _TYPE_DISK, etc */
    gint         id;           /* panel number */

    gint         err;          /* error message logged */

    gint         interval;     /* timer interval */

    guint        handler;      /* timer handler mdh_timeout_del/add() */

    gboolean     hidden;

    gboolean     enabled;

    gchar        *command;     /* click action command */

    gchar        *value;       /* panel value, /var/mail/logan, eth0, etc. */

    gchar        *tooltip;     /* panel tooltip */

    gboolean     bold;         /* threshold reached, bold panel data */

    gpointer     data;
};

struct p_mail_t {
    FILE         *fp;

    GDir         *dp_new;
    GDir         *dp_cur;

    gchar        *md_new;
    gchar        *md_cur;

    gulong       inode;

    gint         type;
    gint         last;

    gint         last_new;
    gint         last_total;
};

#if _MDH_HAS_NET
struct p_net_t {
    FILE         *fp;

    gulong       rx;
    gulong       tx;
};
#endif

struct p_temp_t {
    gint         temp;
    gint         last;
};

struct p_gen_opt_t {
    gboolean     labels;
    gboolean     use_bold;
};

struct p_temp_opt_t {
    gboolean     celcius;      /* display temp in celcius or ferenheight */

    gboolean     proxy_enable;
    gchar        *proxy_host;
    gint         proxy_port;
};

struct toolbar Toolbar;

struct button_t B_Menu, B_Hide, B_Run, B_Pad;

struct custom_t C_User[_MDH_MAX_CUST + 1];

struct panel_t P_Time;

struct panel_t P_Mail[_MDH_MAX_MAIL + 1];
struct p_gen_opt_t P_Mail_opt;

#if _MDH_HAS_CPU
struct panel_t P_Cpu;
struct p_gen_opt_t P_Cpu_opt;
#endif

#if _MDH_HAS_MEM
struct panel_t P_Mem;
struct p_gen_opt_t P_Mem_opt;
#endif

#if _MDH_HAS_DISK
struct panel_t P_Disk[_MDH_MAX_DISK + 1];
struct p_gen_opt_t P_Disk_opt;
#endif

#if _MDH_HAS_NET
struct panel_t P_Net[_MDH_MAX_NET + 1];
struct p_gen_opt_t P_Net_opt;
#endif

#if _MDH_HAS_UPTIME
struct panel_t P_Uptime;
#endif

struct panel_t P_Temp;
struct p_temp_opt_t P_Temp_opt;

#define PANEL(p)  ((struct panel_t *)  p)
#define BUTTON(b) ((struct button_t *) b)
#define CUSTOM(c) ((struct custom_t *) c)

void mdh_toolbar_set_layer_on_top(gboolean state);
