/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "mdh.h"
#include "util.h"

#include "mdh_gen.h"

#include "mdh_toolbar.h"
#include "mdh_custom.h"

extern GtkTooltips *tool_tips;

/* configuring custom buttons */
enum {
    USER_ENABLED,
    USER_NAME,
    USER_TOOLTIP,
    USER_COMMAND,
    USER_EDITABLE,
    USER_N_COLUMNS
};

struct mdh_tree_view_t {
    GtkWidget    *window;
    GtkTreeView  *tree_view;
    GtkTreeModel *tree_model;
};

static void mdh_toolbar_custom_cell_edit(GtkCellRendererText *cell,
                                         const gchar *path_string,
                                         const gchar *new_text,
                                         gpointer data);

static void mdh_toolbar_custom_toggled(GtkCellRendererToggle *cell,
                                       gchar *path_str,
                                       gpointer data);

static void mdh_toolbar_custom_close(struct mdh_tree_view_t *tree);
static void mdh_toolbar_custom_apply(GtkTreeModel *tree_model);
static void mdh_toolbar_custom_cancel(struct mdh_tree_view_t *tree);

static void mdh_toolbar_custom_move_up(struct mdh_tree_view_t *tree);
static void mdh_toolbar_custom_move_down(struct mdh_tree_view_t *tree);

void mdh_toolbar_custom(void)
{
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *main_hbox, *hbox;
    GtkWidget *button;
    GtkWidget *vsep;

    GtkWidget *scrolled_window;

    GtkWidget *tree_view;
    GtkListStore *store;
    GtkCellRenderer *renderer;

    GtkTreeViewColumn *column;

    GtkTreeModel *model;

    GtkTreeIter iter;

    gint i;

    struct mdh_tree_view_t *tree;

    if(Toolbar.custom_open == TRUE)
        return;

    /* expand panel */
    if(Toolbar.hidden)
        mdh_toolbar_toggle_all();

    /* set after mdh_toolbar_toggle_all() otherwise the toolbar won't expand */
    mdh_toolbar_window_toggle_open(&Toolbar.custom_open);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "MailDooHicky: Custom Buttons");
    gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);

    gtk_widget_set_usize(GTK_WIDGET(window), 475, 325);

    g_signal_connect(G_OBJECT(window), "delete_event",
        G_CALLBACK(gtk_true), NULL);

    /* g_signal_connect(G_OBJECT(window), "event",
        G_CALLBACK(mdh_window_event), NULL); */

    gtk_container_set_border_width(GTK_CONTAINER(window), 5);

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(window), vbox);

    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(scrolled_window);

    gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

    store = gtk_list_store_new(USER_N_COLUMNS,
                               G_TYPE_BOOLEAN,
                               G_TYPE_STRING,
                               G_TYPE_STRING,
                               G_TYPE_STRING,
                               G_TYPE_BOOLEAN);

    /* Create a view */
    tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    gtk_widget_show(tree_view);

    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree_view), TRUE);

    gtk_container_add(GTK_CONTAINER(scrolled_window), tree_view);

    g_object_unref(G_OBJECT(store));

    model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));

    tree = g_new(struct mdh_tree_view_t, 1);

    tree->window     = window;
    tree->tree_view  = GTK_TREE_VIEW(tree_view);
    tree->tree_model = model;

    renderer = gtk_cell_renderer_toggle_new();

    g_signal_connect(renderer, "toggled",
        G_CALLBACK(mdh_toolbar_custom_toggled), model);

    /* activate the toggles if enabled */
    for(i = 0; i <= _MDH_MAX_CUST; i++)
        gtk_cell_renderer_toggle_set_active(GTK_CELL_RENDERER_TOGGLE(renderer),
            C_User[i].enabled);

    column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
             "active", USER_ENABLED, NULL);

    gtk_tree_view_column_set_resizable(column, FALSE);
    gtk_tree_view_column_set_min_width(column, 30);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);

    renderer = gtk_cell_renderer_text_new();

    g_signal_connect(renderer, "edited",
        G_CALLBACK(mdh_toolbar_custom_cell_edit), model);

    g_object_set_data(G_OBJECT(renderer), "column", (gint *) USER_NAME);

    column = gtk_tree_view_column_new_with_attributes("Name", renderer,
             "text", USER_NAME, "editable", USER_EDITABLE, NULL);

    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_min_width(column, 50);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);

    renderer = gtk_cell_renderer_text_new();

    g_signal_connect(renderer, "edited",
        G_CALLBACK(mdh_toolbar_custom_cell_edit), model);

    g_object_set_data(G_OBJECT(renderer), "column", (gint *) USER_TOOLTIP);

    column = gtk_tree_view_column_new_with_attributes("Tooltip", renderer,
             "text", USER_TOOLTIP, "editable", USER_EDITABLE, NULL);

    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_min_width(column, 150);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);

    renderer = gtk_cell_renderer_text_new();

    g_signal_connect(renderer, "edited",
        G_CALLBACK(mdh_toolbar_custom_cell_edit), model);

    g_object_set_data(G_OBJECT(renderer), "column", (gint *) USER_COMMAND);

    column = gtk_tree_view_column_new_with_attributes("Command", renderer,
             "text", USER_COMMAND, "editable", USER_EDITABLE, NULL);

    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);

    /* fill in the cells */
    for(i = 0; i <= _MDH_MAX_CUST; i++) {
        gtk_list_store_append(GTK_LIST_STORE(store), &iter);
        gtk_list_store_set(GTK_LIST_STORE(store), &iter,
            USER_ENABLED, C_User[i].enabled,
            USER_NAME,    C_User[i].value,
            USER_TOOLTIP, C_User[i].tooltip,
            USER_COMMAND, C_User[i].command,
            USER_EDITABLE, TRUE, -1);
    }

    main_hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), main_hbox, FALSE, TRUE, 0);
    gtk_widget_show(main_hbox);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start(GTK_BOX(main_hbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show(hbox);

    button = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_tooltips_set_tip(tool_tips, button, "Move selected button up");

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_custom_move_up), tree);

    button = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_tooltips_set_tip(tool_tips, button, "Move selected button down");

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_custom_move_down), tree);

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(main_hbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    vsep = gtk_vseparator_new();
    gtk_box_pack_start(GTK_BOX(hbox), vsep, FALSE, FALSE, 10);
    gtk_widget_show(vsep);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start(GTK_BOX(main_hbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show(hbox);

    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_tooltips_set_tip(tool_tips, button,
        "Cancel new (or unapplied) button settings");

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_custom_cancel), tree);

    button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_tooltips_set_tip(tool_tips, button, "Apply new button settings");

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_custom_apply), model);

    button = gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_tooltips_set_tip(tool_tips, button,
        "Close and Apply new button settings");

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_custom_close), tree);

    gtk_widget_show(window);
}

static void mdh_toolbar_custom_cell_edit(GtkCellRendererText *cell,
                                         const gchar *path_string,
                                         const gchar *new_text,
                                         gpointer data)
{
    GtkTreeModel *model = data;
    GtkTreePath  *path  = gtk_tree_path_new_from_string(path_string);
    GtkTreeIter  iter;

    gchar *old_text, *text = (gchar *) new_text;
    gint *column = g_object_get_data(G_OBJECT(cell), "column");

    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_model_get(model, &iter, column, &old_text, -1);

    switch(GPOINTER_TO_INT(column)) {
        case USER_NAME:
            if(!STRLEN(text)) {
                mdh_window_notice("Name is required.",
                    "This field cannot be empty, reverting to previous.");

                text = old_text;

                break;
            }

            break;

        case USER_COMMAND:
            if(!STRLEN(text)) {
                mdh_window_notice("Command is required.",
                    "This field cannot be empty, reverting to previous.");

                text = old_text;

                break;
            }

            break;

        default:
            break;
    }

    gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, text, -1);

    g_free(old_text);

    gtk_tree_path_free(path);
}

static void mdh_toolbar_custom_toggled(GtkCellRendererToggle *cell,
                                       gchar *path_str,
                                       gpointer data)
{
    GtkTreeModel *model = data;
    GtkTreePath  *path  = gtk_tree_path_new_from_string(path_str);
    GtkTreeIter  iter;

    gboolean enabled;

    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_model_get(model, &iter, USER_ENABLED, &enabled, -1);

    enabled ^= 1;

    gtk_list_store_set(GTK_LIST_STORE(model), &iter, USER_ENABLED, enabled, -1);

    gtk_tree_path_free(path);
}

static void mdh_custom_set_new_value(struct custom_t *obj,
                                     gboolean enabled,
                                     gchar *name,
                                     gchar *tooltip,
                                     gchar *command)
{
    g_free(obj->value);
    g_free(obj->tooltip);
    g_free(obj->command);

    obj->enabled = enabled;

    obj->value   = name;
    obj->tooltip = tooltip;
    obj->command = command;
}

static void mdh_custom_set_new_values(GtkTreeModel *tree_model)
{
    GtkTreeIter iter;

    gboolean enabled;
    gchar *name, *tooltip, *command;

    gint row = 0;

    gtk_tree_model_get_iter_first(tree_model, &iter);

    do {
        if(row > _MDH_MAX_CUST)
            break;

        gtk_tree_model_get(tree_model, &iter, USER_ENABLED, &enabled, -1);
        gtk_tree_model_get(tree_model, &iter, USER_NAME,    &name,    -1);
        gtk_tree_model_get(tree_model, &iter, USER_TOOLTIP, &tooltip, -1);
        gtk_tree_model_get(tree_model, &iter, USER_COMMAND, &command, -1);

        mdh_custom_set_new_value(&C_User[row++],
                                 enabled, name, tooltip, command);
    } while(gtk_tree_model_iter_next(tree_model, &iter));
}

static void mdh_toolbar_custom_close_common(struct mdh_tree_view_t *tree)
{
    gtk_widget_destroy(GTK_WIDGET(tree->window));

    mdh_toolbar_window_toggle_open(&Toolbar.custom_open);

    g_free(tree);
}

static void mdh_toolbar_custom_close(struct mdh_tree_view_t *tree)
{
    mdh_toolbar_custom_apply(tree->tree_model);
    mdh_toolbar_custom_close_common(tree);
}

static void mdh_toolbar_custom_apply(GtkTreeModel *tree_model)
{
    mdh_custom_set_new_values(tree_model);

    /* just do the custom buttons */
    mdh_toolbar_reset_custom_buttons();
}

static void mdh_toolbar_custom_cancel(struct mdh_tree_view_t *tree)
{
    mdh_toolbar_custom_close_common(tree);
}

static void mdh_toolbar_custom_move_up(struct mdh_tree_view_t *tree)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreePath *path;
    GtkTreeIter iter, iter_dst;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree->tree_view));

    if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
        return;

    path = gtk_tree_model_get_path(model, &iter);

    if(gtk_tree_path_prev(path)) {
        gtk_tree_model_get_iter(model, &iter_dst, path);
        gtk_list_store_swap(GTK_LIST_STORE(model), &iter, &iter_dst);
    }

    gtk_tree_path_free(path);
}

#define MDH_VALID_ITER(iter, store) \
    (&iter != NULL && iter.user_data != NULL && store->stamp == iter.stamp)

static void mdh_toolbar_custom_move_down(struct mdh_tree_view_t *tree)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreePath *path;
    GtkTreeIter iter, iter_dst;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree->tree_view));

    if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
        return;

    path = gtk_tree_model_get_path(model, &iter);

    /* why is gtk_tree_path_next void but gtk_tree_path_prev is gboolean? */
    gtk_tree_path_next(path);

    gtk_tree_model_get_iter(model, &iter_dst, path);

    if(MDH_VALID_ITER(iter_dst, GTK_LIST_STORE(model)))
        gtk_list_store_swap(GTK_LIST_STORE(model), &iter, &iter_dst);

    gtk_tree_path_free(path);
}
