/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "mdh.h"

#include "mdh_err.h"
#include "mdh_gen.h"

#include "mdh_toolbar.h"

static void mdh_toolbar_error_log_close(void)
{
    mdh_toolbar_window_toggle_open(&Toolbar.error_open);
}

static void mdh_toolbar_error_log_refresh(GtkListStore *store)
{
    GSList *e;

    g_return_if_fail(Toolbar.errors != NULL && store != NULL);

    /* clear the current list */
    gtk_list_store_clear(store);

    for(e = Toolbar.errors; e != NULL; e = g_slist_next(e)) {
        GtkTreeIter iter;

        gtk_list_store_append(store, &iter);
        gtk_list_store_set(store, &iter, 0, e->data, -1);
    }
}

void mdh_toolbar_error_log(void)
{
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *button;

    GtkWidget *scrolled_window;

    GtkListStore *store;
    GtkWidget *tree;
    GtkCellRenderer *renderer;

    if(Toolbar.error_open == TRUE)
        return;

    mdh_toolbar_window_toggle_open(&Toolbar.error_open);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "MailDooHicky: Error Log");

    gtk_widget_set_usize(GTK_WIDGET(window), 575, 200);

    g_signal_connect(G_OBJECT(window), "destroy",
        G_CALLBACK(mdh_toolbar_error_log_close), NULL);

    g_signal_connect(G_OBJECT(window), "event",
        G_CALLBACK(mdh_window_event), NULL);

    gtk_container_set_border_width(GTK_CONTAINER(window), 5);

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(window), vbox);

    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(scrolled_window);

    gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

    store = gtk_list_store_new(1, G_TYPE_STRING);

    /* Create a view */
    tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    gtk_widget_show(tree);

    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);

    gtk_container_add(GTK_CONTAINER(scrolled_window), tree);

    g_object_unref(G_OBJECT(store));

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), FALSE);

    renderer = gtk_cell_renderer_text_new();

    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree),
        -1, "Log Entry", renderer, "text", 0, NULL);

    mdh_toolbar_error_log_refresh(store);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
    gtk_widget_show(button);

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_error_log_refresh), store);

    button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
    gtk_widget_show(button);

    /* mdh_widget_set_default(GTK_WINDOW(window), button); */

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(gtk_widget_destroy), window);

    gtk_widget_show(window);
}
