/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "mdh.h"

#include "mdh_gen.h"
#include "mdh_pad.h"

#include "mdh_toolbar.h"

struct pad_win_t {
    gint          width;
    gint          height;
    gint          x;
    gint          y;

    gint          last_tab;     /* last tab used in the scratch-pad */

    GtkWidget     *notebook;
    GtkWidget     *text[_MDH_MAX_PAD + 1];
    GtkTextBuffer *buff[_MDH_MAX_PAD + 1];
};

static struct pad_win_t Pad;

static void mdh_toolbar_scratch_pad_clear(GtkWidget *widget,
                                          GtkWidget *notebook);
static void mdh_toolbar_scratch_pad_clear_all(void);
static void mdh_toolbar_scratch_pad_close(void);

static GtkFunction mdh_toolbar_scratch_pad_configure(GtkWidget *widget,
                                                     GdkEventConfigure *event);

void mdh_toolbar_scratch_pad(void)
{
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *vbox2;
    GtkWidget *hbox;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *label;

    gint i;

    if(Toolbar.pad_open == TRUE)
        return;

    mdh_toolbar_window_toggle_open(&Toolbar.pad_open);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "MailDooHicky: Scratch-Pad");
    gtk_widget_set_usize(GTK_WIDGET(window), 300, 300);
    gtk_window_set_default_size(GTK_WINDOW(window), Pad.width, Pad.height);

    if(Pad.x != -1 && Pad.y != -1)
        gtk_widget_set_uposition(window, Pad.x, Pad.y);

    g_signal_connect(G_OBJECT(window), "destroy",
        G_CALLBACK(mdh_toolbar_scratch_pad_close), NULL);

    g_signal_connect(G_OBJECT(window), "configure_event",
        G_CALLBACK(mdh_toolbar_scratch_pad_configure), NULL);

    /* g_signal_connect(G_OBJECT(window), "event",
        G_CALLBACK(mdh_window_event), NULL); */

    gtk_container_set_border_width(GTK_CONTAINER(window), 5);

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(window), vbox);

    Pad.notebook = gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(Pad.notebook), GTK_POS_TOP);
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(Pad.notebook), TRUE);
    gtk_notebook_set_homogeneous_tabs(GTK_NOTEBOOK(Pad.notebook), TRUE);
    gtk_widget_show(Pad.notebook);

    gtk_box_pack_start(GTK_BOX(vbox), Pad.notebook, TRUE, TRUE, 0);

    for(i = 0; i <= _MDH_MAX_PAD; i++) {
        GtkWidget *scrolled_window;

        vbox2 = gtk_vbox_new(FALSE, 0);
        gtk_widget_show(vbox2);

        scrolled_window = gtk_scrolled_window_new(NULL, NULL);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
        gtk_widget_show(scrolled_window);

        table = gtk_table_new(2, 5, FALSE);
        gtk_box_pack_start(GTK_BOX(vbox2), table, TRUE, TRUE, 0);
        gtk_widget_show(table);

        Pad.text[i] = gtk_text_view_new();
        gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(Pad.text[i]), GTK_WRAP_CHAR);
        gtk_text_view_set_editable(GTK_TEXT_VIEW(Pad.text[i]), TRUE);
        gtk_widget_show(Pad.text[i]);

        gtk_container_add(GTK_CONTAINER(scrolled_window), Pad.text[i]);

        gtk_table_attach(GTK_TABLE(table), scrolled_window, 0, 4, 0, 1,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

        /* insert any existing data from a previous use */
        if(Pad.buff[i])
            gtk_text_view_set_buffer(GTK_TEXT_VIEW(Pad.text[i]), Pad.buff[i]);

        button = mdh_button_new_custom_stock(GTK_STOCK_REDO, "Clear");
        gtk_widget_show(button);

        g_signal_connect(G_OBJECT(button), "clicked",
            G_CALLBACK(mdh_toolbar_scratch_pad_clear), Pad.notebook);

        gtk_table_attach(GTK_TABLE(table), button, 0, 2, 1, 2,
            GTK_FILL, GTK_FILL, 0, 0);

        button = mdh_button_new_custom_stock(GTK_STOCK_GO_BACK, "");
        gtk_widget_show(button);

        g_signal_connect_swapped(G_OBJECT(button), "clicked",
            G_CALLBACK(gtk_notebook_prev_page), Pad.notebook);

        gtk_table_attach(GTK_TABLE(table), button, 2, 3, 1, 2,
            GTK_FILL, GTK_FILL, 0, 0);

        button = mdh_button_new_custom_stock(GTK_STOCK_GO_FORWARD, "");
        gtk_widget_show(button);

        g_signal_connect_swapped(G_OBJECT(button), "clicked",
            G_CALLBACK(gtk_notebook_next_page), Pad.notebook);

        gtk_table_attach(GTK_TABLE(table), button, 3, 4, 1, 2,
            GTK_FILL, GTK_FILL, 0, 0);

        {
            gchar *p = g_strdup_printf("Pad %d", i + 1);
            label = gtk_label_new(p);
            g_free(p);
        }

        gtk_notebook_append_page(GTK_NOTEBOOK(Pad.notebook), vbox2, label);
    }

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);
    gtk_widget_show(hbox);

    button = mdh_button_new_custom_stock(GTK_STOCK_REFRESH, "Clear All");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    g_signal_connect(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_scratch_pad_clear_all), NULL);

    button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    /* update scratch-pad buffer data and close the window */
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(gtk_widget_destroy), window);

    /* set the last page used */
    gtk_notebook_set_page(GTK_NOTEBOOK(Pad.notebook), Pad.last_tab);
    gtk_widget_grab_focus(GTK_WIDGET(Pad.text[Pad.last_tab]));

    gtk_widget_show(window);
}

static void mdh_toolbar_scratch_pad_clear(GtkWidget *widget,
                                          GtkWidget *notebook)
{
    gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

    gtk_text_buffer_set_text(Pad.buff[page], "", 0);

    gtk_text_view_set_buffer(GTK_TEXT_VIEW(Pad.text[page]), Pad.buff[page]);
}

static void mdh_toolbar_scratch_pad_clear_all(void)
{
    gint i;

    for(i = 0; i <= _MDH_MAX_PAD; i++) {
        gtk_text_buffer_set_text(Pad.buff[i], "", 0);
        gtk_text_view_set_buffer(GTK_TEXT_VIEW(Pad.text[i]), Pad.buff[i]);
    }
}

static void mdh_toolbar_scratch_pad_close(void)
{
    gint i;

    mdh_toolbar_window_toggle_open(&Toolbar.pad_open);

    Pad.last_tab = gtk_notebook_get_current_page(GTK_NOTEBOOK(Pad.notebook));

    for(i = 0; i <= _MDH_MAX_PAD; i++)
        Pad.buff[i] = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Pad.text[i]));
}

static GtkFunction mdh_toolbar_scratch_pad_configure(GtkWidget *widget,
                                                     GdkEventConfigure *event)
{
    Pad.width  = event->width;
    Pad.height = event->height;

    Pad.x      = event->x;
    Pad.y      = event->y;

    return((GtkFunction) FALSE);
}

void mdh_scratch_pad_init(void)
{
    /* Scratch-Pad geometry */
    Pad.width    = 300;
    Pad.height   = 300;
    Pad.x        = -1;
    Pad.y        = -1;

    Pad.last_tab = 0;

    {
        gint i;

        for(i = 0; i <= _MDH_MAX_PAD; i++)   /* setup scratch-pad buffers */
            Pad.buff[i] = gtk_text_buffer_new(NULL);
    }
}
