/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <unistd.h>

#include <gtk/gtk.h>

#ifdef __linux__
#include <sys/sysinfo.h>
#endif

#ifdef __sun__
#include <sys/loadavg.h>
#endif

#include "mdh.h"
#include "util.h"

#include "mdh_gen.h"

#include "mdh_toolbar.h"
#include "mdh_panel_cpu.h"

#define FIXED_TO_DOUBLE(a) (((double) a / 65536.0))

#if _MDH_HAS_CPU

static const gchar *mdh_toolbar_panel_cpu_get_data(struct panel_t *obj);

static gboolean mdh_toolbar_panel_cpu_get_real(struct panel_t *obj,
                                               gdouble *load);

static GtkFunction mdh_toolbar_panel_cpu_init(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_cpu_init()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) FALSE); */

    if(obj->init == TRUE)
        return((GtkFunction) TRUE);

    obj->init = TRUE;

    return((GtkFunction) TRUE);
}

static GtkFunction mdh_toolbar_panel_cpu_close(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_cpu_close()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    if(obj->init == FALSE)
        return((GtkFunction) FALSE);

    obj->init = FALSE;

    return((GtkFunction) FALSE);
}

static GtkFunction mdh_toolbar_panel_cpu(struct panel_t *obj)
{
    const gchar *text;

    DEBUG(obj, "mdh_toolbar_panel_cpu()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) obj->init); */

    if((text = mdh_toolbar_panel_cpu_get_data(obj)) != NULL)
        mdh_label_set_text(GTK_LABEL(obj->widget), text, obj->bold);

    return((GtkFunction) obj->init);
}

static const gchar *mdh_toolbar_panel_cpu_get_data(struct panel_t *obj)
{
    gdouble load;

    static gchar text[_DATA_MAX];

    DEBUG(obj, "mdh_toolbar_panel_cpu_get_data()");

    g_return_val_if_fail(obj != NULL, NULL);

    obj->bold = FALSE;

    if(obj->init == FALSE) {
        if(obj->init_open(obj) == FALSE) {
            mdh_toolbar_panel_stop(obj);
            return("N/A");
        }
    }

    if(mdh_toolbar_panel_cpu_get_real(obj, &load) == FALSE) {
        mdh_toolbar_panel_stop(obj);
        return("N/A");
    }

    if(P_Cpu_opt.use_bold == TRUE && load >= _BOLD_CPU)
        obj->bold = TRUE;

    g_snprintf(text, sizeof(text), "%3.0f%%", load * 100);

    return(text);
}

static gboolean mdh_toolbar_panel_cpu_get_real(struct panel_t *obj,
                                               gdouble *load)
{
    gdouble l;

#ifdef __linux__
    struct sysinfo sysin;
#else
    gint loadavg;
    gdouble avg[1];
#endif

    DEBUG(obj, "mdh_toolbar_panel_cpu_get_real()");

    g_return_val_if_fail(obj != NULL, FALSE);

#ifdef __linux__ /* getloadavg() opens /proc/loadavg, is this much better? */
    if(sysinfo(&sysin) == -1) {
        PERROR(NULL, 0, "mdh_toolbar_panel_cpu_get_real: sysinfo");
        return(FALSE);
    }

    l = FIXED_TO_DOUBLE(sysin.loads[0]);
#else
    if((loadavg = getloadavg(avg, 1)) == -1) {
        PERROR(NULL, 0, "mdh_toolbar_panel_cpu_get_real: getloadavg");
        return(FALSE);
    }

    if(loadavg > 0)
        l = avg[0];
#endif /* __linux__ */

    if(l > 1)
        l = 1;

    *load = l;

    return(TRUE);
}

void mdh_toolbar_panel_cpu_setup(struct panel_t *obj)
{
    memset(obj, '\0', sizeof(struct panel_t));

    obj->init        = FALSE;
    obj->name        = "cpu";
    obj->command     = g_strdup("xterm -e top");
    obj->tooltip     = g_strdup("Cpu");
    obj->interval    = 3;
    obj->enabled     = TRUE;
    obj->bold        = FALSE;
    obj->callback    = (GtkFunction) mdh_toolbar_panel_command_execute;
    obj->function    = (GtkFunction) mdh_toolbar_panel_cpu;
    obj->init_open   = (GtkFunction) mdh_toolbar_panel_cpu_init;
    obj->init_close  = (GtkFunction) mdh_toolbar_panel_cpu_close;
}

#endif /* _MDH_HAS_CPU */
