/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#include <gtk/gtk.h>

#ifdef __sun__
#include <kstat.h>
#endif

#include "mdh.h"
#include "util.h"

#include "mdh_gen.h"

#include "mdh_toolbar.h"
#include "mdh_panel_net.h"

#if _MDH_HAS_NET

#ifdef __sun__
extern kstat_ctl_t *kc;
#endif

struct net_t {
    gulong rx;
    gulong tx;
};

static const gchar *mdh_toolbar_panel_net_get_data(struct panel_t *obj);

static gboolean mdh_toolbar_panel_net_get_real(struct panel_t *obj,
                                               struct net_t *net);

static GtkFunction mdh_toolbar_panel_net_init(struct panel_t *obj)
{
    struct p_net_t *data;

    DEBUG(obj, "mdh_toolbar_panel_net_init()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) FALSE); */

#ifndef __sun__
    g_return_val_if_fail(obj->value != NULL, (GtkFunction) FALSE);

    if(!STRLEN(obj->value)) {
        ADD_LOG(NULL, 0, "mdh_toolbar_panel_net_init: empty value used");
        mdh_toolbar_panel_stop(obj);
        return((GtkFunction) FALSE);
    }
#endif

    data = obj->data;

    data->rx = data->tx = 0;

#ifdef __sun__ /* FIXME */
    if(P_Net[0].init == TRUE || P_Uptime.init == TRUE)
        return((GtkFunction) TRUE);
#else
    if(obj->init == TRUE)
        return((GtkFunction) TRUE);
#endif

#ifdef __sun__
    if((kc = kstat_open()) == NULL) {
        PERROR(obj, errno, "mdh_toolbar_panel_net_init: kstat_open");
        return((GtkFunction) FALSE);
    }
#else
    if((data->fp = fopen("/proc/net/dev", "r")) == NULL) {
        PERROR(obj, errno, "mdh_toolbar_panel_net_init: fopen: /proc/net/dev");
        return((GtkFunction) FALSE);
    }
#endif

    obj->err   = _ERR_NONE;

#ifdef __sun__ /* FIXME */
    P_Net[0].init = P_Uptime.init = TRUE;
#else
    obj->init  = TRUE;
#endif

    return((GtkFunction) TRUE);
}

static GtkFunction mdh_toolbar_panel_net_close(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_net_close()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

#ifdef __sun__ /* FIXME */
    if(P_Net[0].init == FALSE || P_Uptime.init == FALSE)
        return((GtkFunction) FALSE);
#else
    if(obj->init == FALSE)
        return((GtkFunction) FALSE);
#endif

#ifdef __sun__
    if(kstat_close(kc) == -1)
        PERROR(NULL, 0, "mdh_toolbar_panel_net_close: kstat_close");
#else
    {
        struct p_net_t *data = obj->data;

        fclose(data->fp);
    }
#endif

#ifdef __sun__ /* FIXME */
    P_Net[0].init = P_Uptime.init = FALSE;
#else
    obj->init  = FALSE;
#endif

    return((GtkFunction) FALSE);
}

static GtkFunction mdh_toolbar_panel_net(struct panel_t *obj)
{
    const gchar *text;

    DEBUG(obj, "mdh_toolbar_panel_net()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) obj->init); */

    if((text = mdh_toolbar_panel_net_get_data(obj)) != NULL)
        mdh_label_set_text(GTK_LABEL(obj->widget), text, FALSE);

    return((GtkFunction) obj->init);
}

static const gchar *mdh_toolbar_panel_net_get_data(struct panel_t *obj)
{
    gulong net_rx_last, net_tx_last;
    gdouble n_rx, n_tx;

    static gchar text[_DATA_MAX];

    struct net_t net;

    DEBUG(obj, "mdh_toolbar_panel_net_get_data()");

    g_return_val_if_fail(obj != NULL, NULL);

    if(obj->init == FALSE) {
        if(obj->init_open(obj) == FALSE) {
            mdh_toolbar_panel_stop(obj);
            return("N/A");
        }
    }

    if(mdh_toolbar_panel_net_get_real(obj, &net) == FALSE) {
        mdh_toolbar_panel_stop(obj);

#ifdef __sun__
        g_strlcpy(text, "N/A", sizeof(text));
#else
        if(P_Net_opt.labels == TRUE)
            g_snprintf(text, sizeof(text), "%s: N/A", obj->value);
        else
            g_strlcpy(text, "N/A", sizeof(text));
#endif

        return(text);
    }

    if(net.rx == 0 || net.tx == 0) {
#ifdef __sun__
        g_strlcpy(text, "  ---/  ---", sizeof(text));
#else
        if(P_Net_opt.labels == TRUE)
            g_snprintf(text, sizeof(text), "%s:   ---/  ---", obj->value);
        else
            g_strlcpy(text, "  ---/  ---", sizeof(text));
#endif

        return(text);
    }

    {
        struct p_net_t *data = obj->data;

        net_rx_last = data->rx;
        net_tx_last = data->tx;

        data->rx = net.rx;
        data->tx = net.tx;
    }

    if(net_rx_last == 0 || net_tx_last == 0) {
#ifdef __sun__
        g_strlcpy(text, "  0.0/  0.0", sizeof(text));
#else
        if(P_Net_opt.labels == TRUE)
            g_snprintf(text, sizeof(text), "%s:   0.0/  0.0", obj->value);
        else
            g_strlcpy(text, "  0.0/  0.0", sizeof(text));
#endif

        return(text);
    }

    /* counter wrapping */
    if(net.rx < net_rx_last)
        return(NULL);

    n_rx = net.rx - net_rx_last;
    n_tx = net.tx - net_tx_last;

#ifdef __sun__
    g_snprintf(text, sizeof(text), "%5.1f/%5.1f",
        (n_rx / 1024) / obj->interval, (n_tx / 1024) / obj->interval);
#else
    if(P_Net_opt.labels == TRUE)
        g_snprintf(text, sizeof(text), "%s: %5.1f/%5.1f", obj->value,
            (n_rx / 1024) / obj->interval, (n_tx / 1024) / obj->interval);
    else
        g_snprintf(text, sizeof(text), "%5.1f/%5.1f",
            (n_rx / 1024) / obj->interval, (n_tx / 1024) / obj->interval);
#endif

    return(text);
}

static gboolean mdh_toolbar_panel_net_get_real(struct panel_t *obj,
                                               struct net_t *net)
{
    gulong net_rx, net_tx;

#ifdef __sun__
    kstat_t *ksp;
    kstat_named_t *knp;
#else
    gchar buf[255];
    gboolean interface_found = FALSE;
#endif

    DEBUG(obj, "mdh_toolbar_panel_net_get_real()");

    g_return_val_if_fail(obj != NULL, FALSE);

    net_rx = net_tx = 0;

#ifdef __sun__
    if(kstat_chain_update(kc) == -1) {
        PERROR(NULL, 0, "mdh_toolbar_panel_net_get_data: kstat_chain_update");
        return(FALSE);
    }

    for(ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next) {
        if(STREQ(ksp->ks_class, "net")) {
            kstat_read(kc, ksp, NULL);

            if((knp = kstat_data_lookup(ksp, "rbytes")) == NULL)
                continue;

            net_rx = knp->value.ui32;

            if((knp = kstat_data_lookup(ksp, "obytes")) == NULL)
                continue;

            net_tx = knp->value.ui32;
        }
    }
#else /* ! __sun__ */
{
    struct p_net_t *data = obj->data;

    gint dev_len = STRLEN(obj->value);

    rewind(data->fp);
    fflush(data->fp);

    /* first two lines are headers */
    fgets(buf, sizeof(buf), data->fp);
    fgets(buf, sizeof(buf), data->fp);

    while((fgets(buf, sizeof(buf), data->fp))) {
        gchar *p = g_strchug(buf);

        if(STRNEQ(buf, obj->value, dev_len)) {
            if(sscanf(p + dev_len, ":%lu %*u %*u %*u %*u %*u %*u %*u %lu ",
                &net_rx, &net_tx) == 2) {
                    obj->err = _ERR_NONE;
                    interface_found = TRUE;

                    break;
            }
        }
    }

    if(interface_found == FALSE) {
        ADD_LOG(obj, _ERR_EXIST, "mdh_toolbar_panel_net_get_data: interface (%s) not found",
            obj->value);

        net->rx = net->tx = data->rx = data->tx = 0;

        /* this is no good, interface may come up and down. just let it go */
        /* if(Toolbar.config_open == FALSE)
            mdh_toolbar_panel_stop(obj); */

        return(TRUE);
    }
}
#endif /* __sun__ */

    net->rx = net_rx;
    net->tx = net_tx;

    return(TRUE);
}

void mdh_toolbar_panel_net_setup(struct panel_t *obj,
                                 const gchar *iface,
                                 gboolean enabled)
{
    memset(obj, '\0', sizeof(struct panel_t));

    obj->init        = FALSE;
    obj->name        = "net";
#ifndef __sun__
    obj->value       = (iface) ? g_strdup(iface) : g_strdup("eth0");
    obj->tooltip     = g_strdup_printf("Net: %s", obj->value);
#endif
    obj->command     = g_strdup("ethereal");
    obj->interval    = 3;
    obj->enabled     = enabled;
    obj->bold        = FALSE;
    obj->callback    = (GtkFunction) mdh_toolbar_panel_command_execute;
    obj->function    = (GtkFunction) mdh_toolbar_panel_net;
    obj->init_open   = (GtkFunction) mdh_toolbar_panel_net_init;
    obj->init_close  = (GtkFunction) mdh_toolbar_panel_net_close;

    obj->data        = g_new(struct p_net_t, 1);
}

#endif /* _MDH_HAS_NET */
