/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include <gtk/gtk.h>

#include "mdh.h"
#include "util.h"

#include "mdh_gen.h"

#include "mdh_toolbar.h"
#include "mdh_panel_time.h"

static const gchar *mdh_toolbar_panel_time_get_data(struct panel_t *obj);

static GtkFunction mdh_toolbar_panel_time_init(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_time_init()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) FALSE); */

    if(obj->init == TRUE)
        return((GtkFunction) TRUE);

    g_return_val_if_fail(obj->value != NULL, (GtkFunction) FALSE);

    obj->init = TRUE;

    return((GtkFunction) TRUE);
}

static GtkFunction mdh_toolbar_panel_time_close(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_time_close()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    if(obj->init == FALSE)
        return((GtkFunction) FALSE);

    obj->init = FALSE;

    return((GtkFunction) FALSE);
}

static GtkFunction mdh_toolbar_panel_time(struct panel_t *obj)
{
    const gchar *text;

    DEBUG(obj, "mdh_toolbar_panel_time()");

    g_return_val_if_fail(obj != NULL, (GtkFunction) FALSE);

    /* if(obj->enabled == FALSE)
        return((GtkFunction) obj->init); */

    if((text = mdh_toolbar_panel_time_get_data(obj)) != NULL)
        mdh_label_set_text(GTK_LABEL(obj->widget), text, obj->bold);

    return((GtkFunction) obj->init);
}

static const gchar *mdh_toolbar_panel_time_get_data(struct panel_t *obj)
{
    DEBUG(obj, "mdh_toolbar_panel_time_get_data()");

    g_return_val_if_fail(obj != NULL, NULL);

    if(obj->init == FALSE) {
        if(obj->init_open(obj) == FALSE) {
            mdh_toolbar_panel_stop(obj);
            return("N/A");
        }
    }

    return(mdh_strftime(obj->value));
}

const gchar *mdh_strftime(const gchar *fmt)
{
    static gchar buf[_DATA_MAX];

    time_t time_base;

    g_return_val_if_fail(fmt != NULL, "");

    time_base = time(NULL);

    strftime(buf, sizeof(buf), fmt, localtime(&time_base));

    return(buf);
}

void mdh_toolbar_panel_time_setup(struct panel_t *obj, const gchar *fmt)
{
    memset(obj, '\0', sizeof(struct panel_t));

    obj->init        = FALSE;
    obj->name        = "time";
    obj->value       = (fmt) ? g_strdup(fmt) : g_strdup("%m/%d - %l:%M %p");
    obj->command     = g_strdup("evolution");
    obj->tooltip     = g_strdup("Time");
    obj->interval    = 1;
    obj->enabled     = TRUE;
    obj->bold        = FALSE;
    obj->callback    = (GtkFunction) mdh_toolbar_panel_command_execute;
    obj->function    = (GtkFunction) mdh_toolbar_panel_time;
    obj->init_open   = (GtkFunction) mdh_toolbar_panel_time_init;
    obj->init_close  = (GtkFunction) mdh_toolbar_panel_time_close;
}
