/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "mdh.h"
#include "util.h"

#include "mdh_gen.h"
#include "mdh_run.h"

#include "mdh_toolbar.h"

struct run_win_t {
    GtkWidget *window;
    GtkCombo  *combo;
    gboolean  terminal;
};

static void mdh_toolbar_run_execute(GtkWidget *widget, struct run_win_t *rw);
static void mdh_toolbar_run_close(struct run_win_t *rw);

static void mdh_button_toggle_value(GtkWidget *widget, gboolean *enabled);

static void mdh_toolbar_run_save_toggle(gpointer callback_data,
                                        guint callback_action,
                                        GtkWidget *menu_item)
{
    Toolbar.run_save = GTK_CHECK_MENU_ITEM(menu_item)->active;
}

static void mdh_toolbar_run_close_toggle(gpointer callback_data,
                                         guint callback_action,
                                         GtkWidget *menu_item)
{
    Toolbar.run_close = GTK_CHECK_MENU_ITEM(menu_item)->active;
}

#define _RUN_OPT_HIST  "/Options/Save History"
#define _RUN_OPT_CLOSE "/Options/Close on Execute"

GtkItemFactoryEntry run_menu_items[] = {
    { "/_Options",    NULL, NULL,                         0, "<Branch>" },
    { _RUN_OPT_HIST,  NULL, mdh_toolbar_run_save_toggle,  0, "<ToggleItem>" },
    { _RUN_OPT_CLOSE, NULL, mdh_toolbar_run_close_toggle, 0, "<ToggleItem>" } };

GtkItemFactory *run_item_factory;

static GtkWidget *mdh_toolbar_run_menu_new(void)
{
    run_item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);

    gtk_item_factory_create_items(run_item_factory,
        G_N_ELEMENTS(run_menu_items), run_menu_items, NULL);

    gtk_check_menu_item_set_active(
        GTK_CHECK_MENU_ITEM(
            gtk_item_factory_get_item(run_item_factory, _RUN_OPT_HIST)
        ), Toolbar.run_save);

    gtk_check_menu_item_set_active(
        GTK_CHECK_MENU_ITEM(
            gtk_item_factory_get_item(run_item_factory, _RUN_OPT_CLOSE)
        ), Toolbar.run_close);

    return(gtk_item_factory_get_widget(run_item_factory, "<main>"));
}

void mdh_toolbar_run(void)
{
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *menubar;
    GtkWidget *button;

    struct run_win_t *rw;

    if(Toolbar.run_open == TRUE)
        return;

    mdh_toolbar_window_toggle_open(&Toolbar.run_open);

    rw = g_new(struct run_win_t, 1);

    rw->terminal = FALSE;
    rw->window   = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    rw->combo    = (GtkCombo *) gtk_combo_new();

    gtk_window_set_title(GTK_WINDOW(rw->window), "Run...");
    gtk_window_set_policy(GTK_WINDOW(rw->window), FALSE, FALSE, FALSE);
    gtk_widget_set_usize(GTK_WIDGET(rw->window), 300, 0);

    g_signal_connect_swapped(G_OBJECT(rw->window), "destroy",
        G_CALLBACK(mdh_toolbar_run_close), rw);

    g_signal_connect(G_OBJECT(rw->window), "event",
        G_CALLBACK(mdh_window_event), NULL);

    gtk_container_set_border_width(GTK_CONTAINER(rw->window), 5);

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(rw->window), vbox);

    menubar = mdh_toolbar_run_menu_new();
    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(menubar), FALSE, FALSE, 0);
    gtk_widget_show(menubar);

    gtk_combo_disable_activate(GTK_COMBO(rw->combo));
    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(rw->combo), FALSE, FALSE, 5);
    gtk_widget_show(GTK_WIDGET(rw->combo));

    gtk_entry_set_activates_default(GTK_ENTRY(rw->combo->entry), TRUE);

    gtk_entry_set_max_length(GTK_ENTRY(rw->combo->entry), LINE_MAX);

    if(Toolbar.history != NULL)
        gtk_combo_set_popdown_strings(GTK_COMBO(rw->combo), Toolbar.history);

    button = gtk_check_button_new_with_label("Run in terminal");
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
    gtk_widget_show(button);

    if(!STRLEN(Toolbar.run_item))
        gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

    g_signal_connect(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_button_toggle_value), &rw->terminal);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_widget_show(hbox);

    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    button = mdh_button_new_custom_stock(GTK_STOCK_EXECUTE, "Run");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    mdh_widget_set_default(GTK_WINDOW(rw->window), button);

    g_signal_connect(G_OBJECT(button), "clicked",
        G_CALLBACK(mdh_toolbar_run_execute), rw);

    button = mdh_button_new_custom_stock(GTK_STOCK_CLOSE, "Close");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_widget_show(button);

    g_signal_connect_swapped(G_OBJECT(button), "clicked",
        G_CALLBACK(gtk_widget_destroy), rw->window);

    gtk_widget_show(rw->window);
}

static void mdh_toolbar_run_close(struct run_win_t *rw)
{
    mdh_toolbar_window_toggle_open(&Toolbar.run_open);

    gtk_widget_destroy(rw->window);

    g_object_unref(run_item_factory);

    g_free(rw);
}

static void mdh_toolbar_run_execute(GtkWidget *widget, struct run_win_t *rw)
{
    gchar *command, *text;

    text = g_strdup(gtk_entry_get_text(GTK_ENTRY(rw->combo->entry)));
    text = g_strstrip(text);

    if(!STRLEN(text)) {
        g_free(text);
        return;
    }

    if(rw->terminal == TRUE) {
        gchar tmp[LINE_MAX];

        gint tmp_len = g_snprintf(tmp, sizeof(tmp), "%s -e", Toolbar.run_item);

        /*
         * quick test to see if what's already in the entry contains
         * '<Toolbar.run_item> -e'. probably shouldn't do much more testing
         * in this area, don't want to get annoying.
         */

        if(STRNEQ(text, tmp, tmp_len))
            command = g_strdup(text);
        else
            command = g_strdup_printf("%s -e %s", Toolbar.run_item, text);
    } else {
        command = g_strdup(text);
    }

    g_free(text);

    /* add in reverse order for history dropdown */
    Toolbar.history = g_list_prepend(Toolbar.history, command);

    /* keep run window open if enabled, or execution failed */
    if(mdh_toolbar_exec(command) == FALSE || Toolbar.run_close == FALSE) {
        gtk_combo_set_popdown_strings(GTK_COMBO(rw->combo), Toolbar.history);
        gtk_editable_select_region(GTK_EDITABLE(rw->combo->entry), 0, -1);
    } else {
        gtk_widget_destroy(rw->window);
    }
}

static void mdh_button_toggle_value(GtkWidget *widget, gboolean *enabled)
{
    *enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}
