/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "mdh.h"
#include "util.h"

#include "mdh_toolbar.h"

void set_signal(gint sig, signal_handler handler)
{
    struct sigaction s;

    s.sa_handler = handler;
    s.sa_flags = SA_RESTART;
    sigemptyset(&s.sa_mask);

    sigaction(sig, &s, 0);
}

/* prints an error to stderr and adds it to the error log */
void mdh_error(struct panel_t *obj, gint err, const gchar *fmt, ...)
{
    gchar buf[LINE_MAX];

    va_list args;

    va_start(args, fmt);

    g_vsnprintf(buf, sizeof(buf), fmt, args);

    va_end(args);

    fprintf(stderr, "MailDooHicky: %s\n", buf);

    ADD_LOG(obj, err, "%s", buf);
}

void mdh_perror(struct panel_t *obj, gint err, const gchar *fmt, ...)
{
    gchar buf[LINE_MAX];

    va_list args;

    va_start(args, fmt);

    g_vsnprintf(buf, sizeof(buf), fmt, args);

    va_end(args);

    mdh_error(obj, err, "%s: %s", buf, g_strerror(errno));
}

/* prints debugging info to stderr if enabled */
void mdh_debug(struct panel_t *obj, const gchar *fmt, ...)
{
    gchar buf[LINE_MAX];

    va_list args;

    if(Toolbar.debug == FALSE)
        return;

    va_start(args, fmt);

    g_vsnprintf(buf, sizeof(buf), fmt, args);

    va_end(args);

    if(obj != NULL)
        fprintf(stderr, "%s[%d/%d]: ", obj->name, obj->id, obj->handler);

    fprintf(stderr, "%s\n", buf);
}

void mdh_unlink(const gchar *path)
{
    g_return_if_fail(path != NULL && path[0] != '\0');

    if(unlink(path) == -1)
        PERROR(NULL, 0, "mdh_unlink: %s", path);
}

void mdh_rename(const gchar *src, const gchar *dst)
{
    g_return_if_fail(src != NULL && src[0] != '\0');
    g_return_if_fail(dst != NULL && dst[0] != '\0');

    if(rename(src, dst) == -1)
        PERROR(NULL, 0, "mdh_rename: %s", src);
}
