/*
 * mdh (MailDooHicky)
 *
 * Copyright (c) 2003 Mike Hokenson <mdh at gozer dot org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

typedef void (*signal_handler)(gint);

void set_signal(gint sig, signal_handler handler);

void mdh_perror(struct panel_t *obj, gint err, const gchar *fmt, ...);
void mdh_debug(struct panel_t *obj, const gchar *fmt, ...);

void mdh_unlink(const gchar *path);
void mdh_rename(const gchar *src, const gchar *dst);

#define PERROR           mdh_perror

#define DEBUG            mdh_debug

#ifndef PATH_MAX
# ifdef MAXPATHLEN
#  define PATH_MAX       MAXPATHLEN
# else
#  define PATH_MAX       1024
# endif
#endif

#ifndef LINE_MAX
#define LINE_MAX         2048
#endif

#define ISSET(s)         ((s != NULL && s[0] != '\0'))

#define ATOI(s)          ((ISSET((s)) ? atoi(s)   : 0))
#define STRLEN(s)        ((ISSET((s)) ? strlen(s) : 0))

#define STREQ(s1,s2)     (((ISSET(s1) && ISSET(s2)) && strcmp(s1, s2) == 0))
#define STRNEQ(s1,s2,n)  (((ISSET(s1) && ISSET(s2)) && strncmp(s1, s2, n) == 0))

#define STRUP(s)         ((g_ascii_strup(s, -1)))

#define IS_FILE(s)       ((g_file_test(s, G_FILE_TEST_IS_REGULAR)))
#define IS_LINK(s)       ((g_file_test(s, G_FILE_TEST_IS_SYMLINK)))
#define IS_DIR(s)        ((g_file_test(s, G_FILE_TEST_IS_DIR)))
